% Academic 2D HDG code for solving the Oseen equations with Dirichlet
% boundary conditions.
%
% LaCaN UPC-BarcelonaTech 2017, www.lacan.upc.edu
%
% Main data variables:
%  X: nodal coordinates
%  T: mesh connectivitity matrix
%  F: faces (here sides) for each element
%    (Faces are numbered so that interior faces are first)
%  elemInfo: element type
%  infoFaces.intFaces: [elem1 face1 elem2 face2 rotation] for each face
%    (Each face is assumed to have the orientation given by the
%    first element, it is flipped when seen from the second element)
%  infoFaces.extFaces: [elem1 face1] for each exterior face
%  referenceElement: integration points and shape functions (volume and
%    boundary sides)
%

clearvars, close all, %home
setpath

% Set test case
example = 2; 
%1: analytical flow
%2: kovasznay flow
dir = strcat('oseen/ex_',num2str(example));
addpath(dir)

%% Load computational mesh
degree=2; 
load(['mesh4_P',num2str(degree)]); 
X=2*X-1;
nOfElements = size(T,1);
figure(1),clf,plotMesh(X,T)

%% HDG preprocess
[F, infoFaces] = hdg_preprocess(T);
nOfFaces = max(max(F));

%% Stabilization parameter
tau = ones(nOfElements,3); %Uniform tau parameter (all faces)

%% Computation
% Loop in elements
referenceElement=createReferenceElementTri(degree);
disp('Loop in elements...')
[K, f, localSolverMat, localSolverVec] = hdgMatrixOseen(@viscosityOseen,@advectionOseen,@sourceTermOseen,X,T,F,referenceElement,infoFaces,tau);

%Dirichlet BC
%Dirichlet face nodal coordinates
nOfFaceNodes = degree+1;
nOfInteriorFaces = size(infoFaces.intFaces,1);
nOfExteriorFaces = size(infoFaces.extFaces,1);
uDirichlet=computeProjectionFaces(@analyticalVelocityOseen,infoFaces.extFaces,X,T,referenceElement);
uDirichlet = [uDirichlet;0]; %mean of pressure at last element=0
dofDirichlet=[2*nOfInteriorFaces*nOfFaceNodes + (1:2*nOfExteriorFaces*nOfFaceNodes),2*nOfFaces*nOfFaceNodes+nOfElements];
dofUnknown = [1:2*nOfInteriorFaces*nOfFaceNodes,2*nOfFaces*nOfFaceNodes+(1:nOfElements-1)];
% % System reduction (Dirichlet faces and rho at last element are set to prescribed value)
f = f(dofUnknown)-K(dofUnknown,dofDirichlet)*uDirichlet;
K = K(dofUnknown,dofUnknown);

% Face solution
disp('Solving linear system...')
lambda = K\f;
uhat = [lambda(1:2*nOfInteriorFaces*nOfFaceNodes); uDirichlet];
rho = [lambda(2*nOfInteriorFaces*nOfFaceNodes+1:end) ; 0];

% Elemental solution
disp('Calculating element by element solution...')
[u,L,p]=computeElementsSolutionOseen(uhat,rho,localSolverMat,localSolverVec,F);

% figure(2),clf
% plotDiscontinuosSolution(X,T,u(:,1),referenceElement,20)
% colorbar, title('HDG solution: u1')
% 
% figure(3),clf
% plotDiscontinuosSolution(X,T,u(:,2),referenceElement,20)
% colorbar, title('HDG solution: u2')

figure(4),clf
plotDiscontinuosSolution(X,T,p,referenceElement,20)
colorbar, title('HDG solution: p')

figure(5),clf
plotDiscontinuosSolution(X,T,sqrt(u(:,1).^2+u(:,2).^2),referenceElement,20)
colorbar, title('HDG solution: module of the velocity u')


% Local postprocess for superconvergence
disp('Performing local postprocess...')
referenceElement_star = createReferenceElementTriStar(referenceElement);
u1_star = HDGpostprocess(X,T,u(:,1),L(:,1:2),referenceElement_star);
u2_star = HDGpostprocess(X,T,u(:,2),L(:,3:4),referenceElement_star);

%Plots postprocess solution
% figure(22),clf
% plotPostprocessedSolution(X,T,u1_star,referenceElement_star,20);
% colorbar, title('HDG solution: u1*')
% 
% figure(32),clf
% plotPostprocessedSolution(X,T,u2_star,referenceElement_star,20);
% colorbar, title('HDG solution: u2*')

figure(42),clf
plotPostprocessedSolution(X,T,sqrt(u1_star.^2+u2_star.^2),referenceElement_star,20);
colorbar, title('HDG solution: module of the post-processed velocity u*')



%% Errors

% analytical solution
u1_ex = @analyticalVelocityOseen_u1;
u2_ex = @analyticalVelocityOseen_u2;
p_ex = @analyticalPressureOseen;

%Error for the HDG solution
Error=sqrt(computeL2Norm(referenceElement,X,T,u(:,1),u1_ex)^2+computeL2Norm(referenceElement,X,T,u(:,2),u2_ex)^2);
fprintf('Error HDG velocity = %e\n',Error);

% Error for the postprocessed solution
ErrorPost=sqrt(computeL2NormPostprocess(referenceElement_star,X,T,u1_star,u1_ex)^2+computeL2NormPostprocess(referenceElement_star,X,T,u2_star,u2_ex)^2);
fprintf('Error HDG postprocessed veclocity= %e\n',ErrorPost);

ErrorP=sqrt(computeL2Norm(referenceElement,X,T,p,p_ex)^2);
fprintf('Error HDG pressure = %e\n',ErrorP);
disp(' ')